/*
 * fs_test.c
 *
 *  Created on: 7 Nov 2019
 *      Author: cedric
 */

#include "fs_test.h"
#include <ti/drivers/net/wifi/fs.h>
#include <ti/display/Display.h>

extern Display_Handle display;

// Write a file to file system
// if it doesn't exist, it will be created
long FSWriteFile(unsigned char* filename, unsigned long max_size, unsigned char* buf, int filesize, unsigned char *signature, uint32_t signatureSize, int flag)
{
    long DeviceFileHandle = -1;
    long lRetVal = -1;
    SlFsFileInfo_t FileInfo;

    lRetVal = sl_FsGetInfo(filename, 0, &FileInfo);

    UART_PRINT("File getinfo=%d, FileLen=%lu, AllocatedLen=%lu, WriteCounter=%lu, Flags=%ld", lRetVal, FileInfo.Len, FileInfo.MaxSize, FileInfo.WriteCounter, FileInfo.Flags);

    //TODO: RPD-48: Check if SL_ERROR_FS_FILE_HAS_NOT_BEEN_CLOSE_CORRECTLY needs sl_stop/sl_start to correct
    if (lRetVal == SL_ERROR_FS_FILE_NOT_EXISTS || lRetVal == SL_ERROR_FS_FILE_HAS_NOT_BEEN_CLOSE_CORRECTLY)
    {
        UART_PRINT("File does not exist, creating new file %s with allocated size = %d",
                   filename, max_size);
        DeviceFileHandle = sl_FsOpen(filename,(SL_FS_CREATE | SL_FS_CREATE_MAX_SIZE( max_size ) | flag), 0);

        if (DeviceFileHandle < 0)
        {
            ERROR_PRINT("%d:", DeviceFileHandle);
            return DeviceFileHandle;
        }
    }
    else
    {
//        DBG_PRINT("Opening file ");
        DeviceFileHandle = sl_FsOpen(filename,(SL_FS_WRITE | SL_FS_CREATE_MAX_SIZE( max_size ) | flag), 0);
        if (DeviceFileHandle < 0)
        {
            ERROR_PRINT("%d:", DeviceFileHandle);
            return DeviceFileHandle;
        }
    }

//    DBG_PRINT("Writing file ");
    lRetVal = sl_FsWrite(DeviceFileHandle, 0, (unsigned char *) buf, filesize);
    if (lRetVal < 0)
    {
        sl_FsClose(DeviceFileHandle, 0, 0, 0);
        ERROR_PRINT("%d:", lRetVal);
        return lRetVal;
    }


//    DBG_PRINT("Closing file ");
    lRetVal = sl_FsClose(DeviceFileHandle, NULL, (unsigned char *)signature, signatureSize);
    if (lRetVal < 0)
    {
        ERROR_PRINT("%d:", lRetVal);
        return lRetVal;
    }

//    DBG_PRINT("File closed ");
    return lRetVal;
}

// Read or Write a file from file system
// Return the number of bytes read, 0 if the file does not exist
// if filesize is 0, it will read the whole file
long FSReadFile(unsigned char* filename, unsigned char* buf, int filesize)
{
    long DeviceFileHandle = -1;
    long lRetVal = -1;
    long nbread = 0;
    SlFsFileInfo_t FileInfo;


    lRetVal = sl_FsGetInfo(filename, 0, &FileInfo);

    DBG_PRINT("File getinfo=%d, FileLen=%lu, AllocatedLen=%lu, WriteCounter=%lu, Flags=%ld", lRetVal, FileInfo.Len, FileInfo.MaxSize, FileInfo.WriteCounter, FileInfo.Flags);

    if (lRetVal == SL_ERROR_FS_FILE_NOT_EXISTS)
    {
        UART_PRINT("File %s does not exist", filename);
        return 0;
    }
    if (lRetVal < 0)
    {
        ERROR_PRINT("%d:", lRetVal);
        return lRetVal;
    }

    if (filesize == 0)
    {
        filesize = FileInfo.Len;
    }
    if (FileInfo.Len > filesize)
    {
        return -55;
    }


    if (filesize == 0)
    {
        return 0;
    }
    else
    {
//        DBG_PRINT("Opening file");
        DeviceFileHandle = sl_FsOpen(filename, SL_FS_READ, 0);
        if (DeviceFileHandle < 0)
        {
            ERROR_PRINT("%d:", DeviceFileHandle);
            return DeviceFileHandle;
        }

//        DBG_PRINT("Reading file");
        lRetVal = sl_FsRead(DeviceFileHandle, 0, buf, filesize);
        if (lRetVal < 0)
        {
            ERROR_PRINT("%d:", lRetVal);
            sl_FsClose(DeviceFileHandle, NULL, NULL, 0);
            return lRetVal;
        }

        if (lRetVal > 0)
            nbread = lRetVal;

//        UART_PRINT("Closing file");
        lRetVal = sl_FsClose(DeviceFileHandle, NULL, NULL, 0);

        if (lRetVal < 0)
        {
            ERROR_PRINT("%d:", lRetVal);
            return lRetVal;
        }
        else
            return nbread;
    }
}

long FSWriteTestFile()
{
    Display_printf(display, 0, 0, "Testing\n\r");

//    unsigned char* servicePackImageSig;
//    int servicePackLen;
    long lRetVal = -1;
    unsigned char buffer[500];
    int filesize = 0;
    int len;


    UART_PRINT("Read file original");
    Display_printf(display, 0, 0, "Testing\n\r");

    filesize = FSReadFile("msp_measure.bin", buffer, 0);
    if (filesize < 0)
    {
        ERROR_PRINT("%d", filesize);
    }
    UART_PRINT("Buffer[0] = %d", buffer[0]);

    UART_PRINT("Read file new");
    filesize = FSReadFile("msp_measure_copy.bin", buffer, 0);
    if (filesize < 0)
    {
        ERROR_PRINT("%d", filesize);
    }

    UART_PRINT("Buffer[0] = %d", buffer[0]);
    // extract signature, last bytes from the back of the file
    len = filesize - SIGNATURE_SIZE;
//
//    servicePackImageSig = &servicePack[servicePackLen];
//    UART_PRINT("signature:");
//    for (int i=0; i<SERVICE_PACK_SIG_SIZE; i++)
//    {
//        UART_PRINT_RAW("%x", servicePackImageSig[i]);
//    }

//    DBG_PRINT("Writing file %s %d bytes", SP_FILE_NAME, FileLen);


    UART_PRINT("Write file");
    uint32_t flags = SL_FS_CREATE_PUBLIC_WRITE;
//            SL_FS_OVERWRITE;
//    flags |= SL_FS_CREATE_PUBLIC_WRITE;
//    flags |= SL_FS_CREATE_PUBLIC_READ;
    flags |= SL_FS_CREATE_SECURE;

    lRetVal = FSWriteFile((unsigned char *) "msp_measure.bin", 7500, buffer, len, &buffer[len], SIGNATURE_SIZE, flags);
    if (lRetVal < 0)
    {
        ERROR_PRINT("%d", lRetVal);
    }


    UART_PRINT("Read file");
    filesize = FSReadFile("msp_measure.bin", buffer, 0);
    if (filesize < 0)
    {
        ERROR_PRINT("%d", filesize);
    }
    UART_PRINT("Buffer[0] = %d", buffer[0]);

    return lRetVal;
}



